///File : LongestWord.cpp
//------------------------------------------------------------------------------
/*
    This file is part of LongestWord.

    LongestWord is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    LongestWord is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with LongestWord.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "LongestWord.hpp"
//------------------------------------------------------------------------------

LongestWord::LongestWord() : LetterGameBase()
{
    m_userScore = 0;
    m_pcScore = 0;
    m_LongestWordLen = 0;
    m_isInGame = false;
}
//------------------------------------------------------------------------------
wxChar LongestWord::GetOneLetterHasard(const bool &isVoyelle, const bool &isConsonne) const
{
    if(isVoyelle || isConsonne)
    {
        wxString letters;
        if(isVoyelle)
        {
            letters << _T("AEIOUY");
        }

        if(isConsonne)
        {
            letters << _T("BCDFGHJKLMNPQRSTVWXZ");
        }

        return letters[Hasard(0, letters.Len() - 1)];
    }
    else
    {
        return 'A';
    }
}
//------------------------------------------------------------------------------
std::vector<wxString> LongestWord::GetLongestWordsForCurrentLetters(const bool &longestOnly, const bool &onlyOne) const
{
    std::vector<wxString> longestWordsTbl;

    // On commence pas à HASARD_LETTERS_NB car les mots peuvent être plus long (tiret...)
    for(int nb = m_LongestWordLen; nb >= 0; nb--) // recherche pour les mots de nb lettres
    {
        for(int i = 0; i < m_wordsTbl.size(); i++)
        {
            // Si il n'a pas une taille de nb ==> Pas possible;
            if(m_wordsTbl[i].Len() != nb)
            {
                continue;
            }

            // On vérifie
            if(WordUsesCurrentLetters(m_wordsTbl[i]))
            {
                longestWordsTbl.push_back(m_wordsTbl[i]);

                if(onlyOne)
                {
                    return longestWordsTbl;
                }
            }
        }

        // Si on ne demande que les plus longues et que le tableau contient des entrées pour nb chiffre ==> stop
        if(longestOnly && longestWordsTbl.size() > 0)
        {
            return longestWordsTbl;
        }
    }

    return longestWordsTbl;
}
//------------------------------------------------------------------------------
bool LongestWord::IsInGame() const
{
    return m_isInGame;
}
//------------------------------------------------------------------------------
void LongestWord::NewLetters()
{
    m_currentLetters = _T("");

    bool isConsonne = true;
    for(int i = 0; i < HASARD_LETTERS_NB; i++)
    {
        m_currentLetters << GetOneLetterHasard(!isConsonne, isConsonne);

        isConsonne = !isConsonne;
    }

    m_isInGame = true;
}
//------------------------------------------------------------------------------
bool LongestWord::WordExists(const wxString &word) const
{
    bool testResult = true;

    for(int i = 0; i < m_wordsTbl.size(); i++)
    {
        testResult = true;

        // Si ils ont pas la même taille ==> Pas possible;
        if(m_wordsTbl[i].Len() != word.Len())
        {
            continue;
        }

        // On vérifie lettre par lettre...
        for(int j = 0; j < word.Len(); j++)
        {
            if(GetLettersFor(word[j]).Find(m_wordsTbl[i][j]) == wxNOT_FOUND)
            {
                testResult = false;
                break;
            }
        }

        if(!testResult) // Si pas même lettre
        {
            continue;
        }

        return true;
    }
    return false;
}
//------------------------------------------------------------------------------
bool LongestWord::WordUsesCurrentLetters(const wxString &word, wxString *unusedLetters) const
{
    wxString tmpLettersList = m_currentLetters;
    bool testResult = false;
    bool completeResult = true;

    for(int i = 0; i < word.Len(); i++)
    {
        testResult = false;

        // Test de la lettre i

        for(int j = 0; j < tmpLettersList.Len(); j++)
        {
            // Test de la lettre i avec la lettre j de la liste...
            if(GetLettersFor(tmpLettersList[j]).Find(word[i]) != wxNOT_FOUND)
            {
                // La lettre est trouvée, on la sipprime de la liste et on quitte la boucle pour la prochaine lettre...
                tmpLettersList = tmpLettersList.Mid(0, j) + tmpLettersList.Mid(j+1);
                testResult = true;
                break;
            }
        }

        if(testResult)
        {
            continue;
        }

        // Si on arrive ici, c'est que la lettre en cours (i) n'est pas trouvée...
        // ==> Derniére chance : si elle ne fait pas partie des caractére de l'utilisateur (comme "-") ==> Accépté.

        if(GetEveryUserLetter().Find(word[i]) == wxNOT_FOUND)
        {
            // Ok ==> Lettre suivante
            continue;
        }

        // Si on arrive là : les tests sont mauvais : la lettre ne passe pas !
        completeResult = false;
        if(unusedLetters == NULL) // On arrete là seulement si on ne demande pas les lettres restantes.
        {
            break;
        }
    }

    // Si demandé, on donne les lettres innutilisée
    if(unusedLetters != NULL)
    {
        *unusedLetters = tmpLettersList;
    }

    return completeResult;
}
//------------------------------------------------------------------------------
int LongestWord::GetUserScore() const
{
    return m_userScore;
}
//------------------------------------------------------------------------------
int LongestWord::GetPcScore() const
{
    return m_pcScore;
}
//-----------------------------------------------------------------------------
wxString LongestWord::GetCurrentLetters() const
{
    return m_currentLetters;
}
//-----------------------------------------------------------------------------
wxString LongestWord::GetCurrentWord() const
{
    return m_currentWord;
}
//-----------------------------------------------------------------------------
int LongestWord::GetTimeLeft() const
{
    return 10;
}
//-----------------------------------------------------------------------------
void LongestWord::SetCurrentWord(const wxString &word)
{
    m_currentWord = word;
}
//-----------------------------------------------------------------------------
bool LongestWord::Correct(int *userPointPtr, int *pcPointPtr, std::vector<wxString> *longestWordListPtr)
{
    bool userWordOk = (WordUsesCurrentLetters(m_currentWord) && WordExists(m_currentWord));
    int userPoint, pcPoint;

    if(userWordOk)
    {
        userPoint = m_currentWord.Len();
    }
    else
    {
        userPoint = 0;
    }

    std::vector<wxString> longestWordList = GetLongestWordsForCurrentLetters(true, false);

    pcPoint = longestWordList[0].Len(); // ==> PC a la taille du plus long mot comme pts;

    m_pcScore += pcPoint;
    m_userScore += userPoint;

    if(userPointPtr != NULL)
    {
        *userPointPtr = userPoint;
    }

    if(pcPointPtr != NULL)
    {
        *pcPointPtr = pcPoint;
    }

    if(pcPointPtr != NULL)
    {
        *pcPointPtr = pcPoint;
    }

    if(longestWordListPtr != NULL)
    {
        longestWordListPtr->swap(longestWordList); // On echange les données pour envoyer l'inutilisé à la poubelle.
    }

    m_isInGame = false;

    return userWordOk;
}
//-----------------------------------------------------------------------------
