///File : MainPanel.cpp
//------------------------------------------------------------------------------
/*
    This file is part of LongestWord.

    LongestWord is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    LongestWord is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with LongestWord.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "MainPanel.hpp"
//------------------------------------------------------------------------------
MainPanel::MainPanel( wxWindow* parent, wxWindowID id, LongestWord *hangMan, const wxPoint& pos, const wxSize& size, long style ) : wxPanel( parent, id, pos, size, style )
{
    // LongestWord
    m_LongestWord = hangMan;

    // Controls
	wxBoxSizer *MainSizer = new wxBoxSizer( wxHORIZONTAL );

    wxBoxSizer *LettersWordSizer = new wxBoxSizer( wxVERTICAL );

    // 1 : Mot
    LettersWordSizer->AddStretchSpacer(1);

    wxBoxSizer *WordSizer = new wxBoxSizer( wxHORIZONTAL );

    m_CurrentWord = new wxTextCtrl(this, wxID_ANY, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_CENTRE | wxTE_PROCESS_ENTER);
    m_CurrentWord->SetFont(wxFont(36, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL));
    WordSizer->Add(m_CurrentWord, 1,  wxALL, 5);

    LettersWordSizer->Add(WordSizer, 0, wxEXPAND, 0);

    LettersWordSizer->AddStretchSpacer(1);

    // 2 : Letters
    wxGridSizer *LettersSizer = new wxGridSizer(1, 9, 5, 5);

    for(int i = 0; i < HASARD_LETTERS_NB; i++)
    {
        m_LettersTbl[i] = new wxButton(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(-1, 40));

        LettersSizer->Add(m_LettersTbl[i], 0, wxEXPAND, 0);

        m_LettersTbl[i]->Connect( wxID_ANY, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainPanel::OnLetterBt), NULL, this);
        m_LettersTbl[i]->Connect( wxID_ANY, wxEVT_KEY_DOWN, wxKeyEventHandler(MainPanel::OnKeyPress), NULL, this);
    }
    LettersWordSizer->Add(LettersSizer, 0, wxEXPAND | wxALL, 5);

	MainSizer->Add(LettersWordSizer, 1, wxEXPAND, 0);

	// Line
    MainSizer->Add(new wxStaticLine(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxLI_VERTICAL), 0, wxEXPAND | wxTOP | wxBOTTOM | wxRIGHT, 5 );

	// Partie droite : dessin + text + newBt
	wxSizer *ScoreSizer = new wxBoxSizer( wxVERTICAL );

    ScoreSizer->AddStretchSpacer(1);

    m_Score = new wxStaticText(this, wxID_ANY, _T(""));
    ScoreSizer->Add(m_Score, 0, wxALL, 5);

    #if defined(__WXMSW__) // Bug pour Windows : dès qu'on appuit sur n ==> Valider.
        m_NewWordBt = new wxButton(this, wxID_NEW, _("New"), wxDefaultPosition, wxSize(130, 40));
    #else
        m_NewWordBt = new wxButton(this, wxID_NEW, _T(""), wxDefaultPosition, wxSize(130, 40));
    #endif //#ifdef WIN32
    ScoreSizer->Add(m_NewWordBt, 0, wxALL | wxEXPAND, 5);

    #if defined(__WXMSW__) // Bug pour Windows : dès qu'on appuit sur n ==> Valider.
        m_ValidateBt = new wxButton(this, wxID_OK, _("Ok"), wxDefaultPosition, wxSize(130, 40));
    #else
        m_ValidateBt = new wxButton(this, wxID_OK, _T(""), wxDefaultPosition, wxSize(130, 40));
    #endif //#ifdef WIN32
    ScoreSizer->Add(m_ValidateBt, 0, wxALL | wxEXPAND, 5);

    MainSizer->Add(ScoreSizer, 0, wxEXPAND);

    // Finition
	SetSizer(MainSizer);

	// Events
	m_NewWordBt->Connect( wxID_ANY, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainPanel::OnNewWordBt), NULL, this);
    m_NewWordBt->Connect( wxID_ANY, wxEVT_KEY_DOWN, wxKeyEventHandler(MainPanel::OnKeyPress), NULL, this);

	m_ValidateBt->Connect( wxID_ANY, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainPanel::OnValidateBt), NULL, this);
    m_ValidateBt->Connect( wxID_ANY, wxEVT_KEY_DOWN, wxKeyEventHandler(MainPanel::OnKeyPress), NULL, this);

	m_CurrentWord->Connect( wxID_ANY, wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler(MainPanel::OnWordUpdate), NULL, this);
	m_CurrentWord->Connect( wxID_ANY, wxEVT_COMMAND_TEXT_ENTER, wxCommandEventHandler(MainPanel::OnValidateBt), NULL, this);
    // Update info
    NewLetters();
}
//------------------------------------------------------------------------------
void MainPanel::OnWordUpdate(wxCommandEvent& event)
{
    if(m_LongestWord->WordUsesCurrentLetters(m_CurrentWord->GetValue()))
    {
        m_LongestWord->SetCurrentWord(m_CurrentWord->GetValue());
    }
    else
    {
        m_CurrentWord->SetValue(m_LongestWord->GetCurrentWord());
        wxBell();
    }

    MyRefresh();
}
//------------------------------------------------------------------------------
void MainPanel::OnKeyPress(wxKeyEvent& event)
{
    if( m_LongestWord->IsInGame() )
    {
        for(int i = 0; i < HASARD_LETTERS_NB; i++)
        {
            if(m_LettersTbl[i]->IsEnabled() && event.GetKeyCode() == m_LettersTbl[i]->GetLabel()[0]) // Si le bt est activé et que c'est la bonne lettre...
            {
                // Refresh & co auto : on modifie le texte.
                m_CurrentWord->SetValue(m_CurrentWord->GetValue() + wxChar(event.GetKeyCode()));

                m_CurrentWord->SetFocus();
                m_CurrentWord->SetInsertionPointEnd();

                break;
            }
        }

        // Back
        if(event.GetKeyCode() == WXK_BACK && m_CurrentWord->GetValue().Len() > 0)
        {
            // Refresh & co auto : on modifie le texte.

            m_CurrentWord->SetValue(m_CurrentWord->GetValue().Mid(0, m_CurrentWord->GetValue().Len() - 1));

            m_CurrentWord->SetFocus();
            m_CurrentWord->SetInsertionPointEnd();
        }
    }


    event.Skip();
}
//------------------------------------------------------------------------------
void MainPanel::OnLetterBt(wxCommandEvent& event)
{
    wxChar letter = '\0';

    for(int i = 0; i < HASARD_LETTERS_NB; i++)
    {
        if(event.GetId() == m_LettersTbl[i]->GetId())
        {
            letter = m_LettersTbl[i]->GetLabel()[0];

            m_CurrentWord->SetValue(m_CurrentWord->GetValue() + letter);

            m_CurrentWord->SetFocus();
            m_CurrentWord->SetInsertionPointEnd();

            break;
        }
    }
}
//------------------------------------------------------------------------------
void MainPanel::OnNewWordBt(wxCommandEvent& event)
{
    NewLetters();
}
//------------------------------------------------------------------------------
void MainPanel::OnValidateBt(wxCommandEvent& event)
{
    if( !m_LongestWord->IsInGame() )
    {
        return;
    }

    wxTopLevelWindow *mainFrame = static_cast<wxTopLevelWindow*>(GetParent());

    wxString frameTitle = mainFrame->GetTitle();
    wxString frameWaitTitle = frameTitle + _T(" - ") + _("Please wait...");

    mainFrame->SetTitle(frameWaitTitle);
    wxBeginBusyCursor();


    wxString msg;

    int userPt, pcPt;
    std::vector<wxString> longestWordList;
    bool wordOk = m_LongestWord->Correct(&userPt, &pcPt, &longestWordList);

    if(!longestWordList.empty())
    {
        if(wordOk && userPt == pcPt && longestWordList.size() == 1) // Le plus long mot
        {
            msg << _("Well done, you have found the longest word !");
        }
        else if(wordOk && userPt == pcPt)
        {
            msg << _("Congratulation, you have found one of the longest word !");
        }
        else if(wordOk)
        {
            msg << _("The computer has found a longest word than you !");
        }
        else if(!wordOk && !m_LongestWord->WordUsesCurrentLetters(m_LongestWord->GetCurrentWord()))
        {
            msg << _("Your word use wrong letters ! You should only use given letters and special letter ('-', '_', ' '...)");
        }
        else
        {
            msg << _("Your word doesn't exist !");
        }
        msg << _T("\n");
        msg << wxString::Format(_("You own %d points and the computer owns %d points."), userPt, pcPt);
        msg << _T("\n\n");
        msg << wxString::Format(_("These words were composed of %d letters:"), longestWordList[0].Len());

        for(int i = 0; i < longestWordList.size(); i++)
        {
            msg << _T("\n - ");
            msg << longestWordList[i];
        }

        msg << _T("\n\n");
        msg << _("Do you want to see the whole word list ?");

        wxEndBusyCursor();
        mainFrame->SetTitle(frameTitle);

        if(wxMessageBox(msg, _("Results"), wxYES_NO | wxNO_DEFAULT, this) == wxYES)
        {
            wxBeginBusyCursor();
            mainFrame->SetTitle(frameWaitTitle);
            mainFrame->Update();


            longestWordList = m_LongestWord->GetLongestWordsForCurrentLetters(false, false);

            // Rajout des tags pour le nombre de lettre.
            int lastLetterNb = 0;
            for(int i = 0; i < longestWordList.size(); i++)
            {
                if(lastLetterNb != longestWordList[i].Len())
                {
                    longestWordList.insert(longestWordList.begin() + i, wxString::Format(_("--- %d letters: ---"), longestWordList[i].Len()));
                    i++; // On passe la chaine qu'on a ajouté.
                    lastLetterNb = longestWordList[i].Len();
                }
            }


            wxString *longestWordCList = new wxString[longestWordList.size()];
            copy( longestWordList.begin(), longestWordList.end(), longestWordCList);

            wxEndBusyCursor();
            mainFrame->SetTitle(frameTitle);

            wxGetSingleChoice(_("Whole word list:"), _("Whole word list"), longestWordList.size(), longestWordCList, this);
            delete [] longestWordCList;
        }
    }
    else
    {
        msg = _("There were no word for these letters !");
    }

    mainFrame->SetTitle(frameTitle);
    wxEndBusyCursor();

    MyRefresh();
}
//------------------------------------------------------------------------------
void MainPanel::NewLetters()
{
    m_CurrentWord->SetValue(_T(""));
    m_LongestWord->NewLetters();

    // On remplie les boutons...
    for(int i = 0; i < HASARD_LETTERS_NB; i++)
    {
        m_LettersTbl[i]->SetLabel(m_LongestWord->GetCurrentLetters()[i]);
    }

    MyRefresh();
    m_CurrentWord->SetFocus();
}
//------------------------------------------------------------------------------
void MainPanel::MyRefresh()
{
    if(!m_LongestWord->IsInGame())
    {
        // Désactivation de tous les bouttons
        for(int i = 0; i < HASARD_LETTERS_NB; i++)
        {
            m_LettersTbl[i]->Disable();
        }

        m_ValidateBt->Disable();

        m_CurrentWord->SetEditable(false);
        m_CurrentWord->SetBackgroundColour(*wxWHITE);

        // Focus pour le Bt nouveau
        m_NewWordBt->SetFocus();
    }
    else
    {
        m_CurrentWord->SetEditable(true);

        // Désactivation des bouttons inutils
        wxString hasardLettersLeft;
        bool wordIsOk;

        wordIsOk = m_LongestWord->WordUsesCurrentLetters(m_LongestWord->GetCurrentWord(), &hasardLettersLeft);

        for(int i = 0; i < HASARD_LETTERS_NB; i++)
        {
            int findResult = hasardLettersLeft.Find(m_LettersTbl[i]->GetLabel()[0]);

            if(findResult != wxNOT_FOUND) // Si la lettre n'est pas encore utilisée
            {
                m_LettersTbl[i]->Enable();

                // On supprime cette lettre de la liste au cas ou il y ai deux fois une même lettre...
                hasardLettersLeft = hasardLettersLeft.Mid(0, findResult) + hasardLettersLeft.Mid(findResult + 1);
            }
            else
            {
                m_LettersTbl[i]->Disable();
            }
        }

        // Couleur
        if(m_LongestWord->GetCurrentWord() != _T("")) // Pas vide
        {
            if(wordIsOk)
            {
                m_CurrentWord->SetBackgroundColour(*wxGREEN);
            }
            else
            {
                m_CurrentWord->SetBackgroundColour(*wxRED);
            }
        }
        else
        {
            m_CurrentWord->SetBackgroundColour(*wxWHITE);
        }

        m_ValidateBt->Enable();
    }

    // Score
    m_Score->SetLabel(wxString::Format(_("Score:\n - You: %d\n - Computer: %d"), m_LongestWord->GetUserScore(), m_LongestWord->GetPcScore()));

    Layout();
}
